@@ -18,7 +18,6 @@
 
 import netaddr
 from ryu.lib import hub
-from ryu.base import app_manager
 from ryu.services.protocols.bgp.operator import ssh
 
 from ryu.services.protocols.bgp.core_manager import CORE_MANAGER
@@ -27,6 +26,7 @@ from ryu.services.protocols.bgp.api.base import call
 from ryu.services.protocols.bgp.api.base import PREFIX
 from ryu.services.protocols.bgp.api.base import NEXT_HOP
 from ryu.services.protocols.bgp.api.base import ROUTE_DISTINGUISHER
+from ryu.services.protocols.bgp.api.base import ROUTE_FAMILY
 from ryu.services.protocols.bgp.rtconf.common import LOCAL_AS
 from ryu.services.protocols.bgp.rtconf.common import ROUTER_ID
 from ryu.services.protocols.bgp.rtconf.common import BGP_SERVER_PORT
@@ -56,10 +56,13 @@ from ryu.services.protocols.bgp.rtconf.neighbors import PEER_NEXT_HOP
 from ryu.services.protocols.bgp.rtconf.neighbors import PASSWORD
 from ryu.services.protocols.bgp.rtconf.neighbors import IN_FILTER
 from ryu.services.protocols.bgp.rtconf.neighbors import OUT_FILTER
+from ryu.services.protocols.bgp.rtconf.neighbors import IS_ROUTE_SERVER_CLIENT
 from ryu.services.protocols.bgp.info_base.base import Filter
 
 
 NEIGHBOR_CONF_MED = 'multi_exit_disc'
+RF_VPN_V4 = vrfs.VRF_RF_IPV4
+RF_VPN_V6 = vrfs.VRF_RF_IPV6
 
 
 class EventPrefix(object):
@@ -93,7 +96,8 @@ class BGPSpeaker(object):
                  refresh_stalepath_time=DEFAULT_REFRESH_STALEPATH_TIME,
                  refresh_max_eor_time=DEFAULT_REFRESH_MAX_EOR_TIME,
                  best_path_change_handler=None,
-                 ssh_console=False):
+                 ssh_console=False,
+                 label_range=DEFAULT_LABEL_RANGE):
         """Create a new BGPSpeaker object with as_number and router_id to
         listen on bgp_server_port.
 
@@ -130,14 +134,12 @@ class BGPSpeaker(object):
         settings[BGP_SERVER_PORT] = bgp_server_port
         settings[REFRESH_STALEPATH_TIME] = refresh_stalepath_time
         settings[REFRESH_MAX_EOR_TIME] = refresh_max_eor_time
+        settings[LABEL_RANGE] = label_range
         self._core_start(settings)
         self._init_signal_listeners()
         self._best_path_change_handler = best_path_change_handler
-
         if ssh_console:
-            app_mgr = app_manager.AppManager.get_instance()
-            ssh_cli = app_mgr.instantiate(ssh.Cli)
-            ssh_cli.start()
+            hub.spawn(ssh.SSH_CLI_CONTROLLER.start)
 
     def _notify_best_path_changed(self, path, is_withdraw):
         if not path.source:
@@ -177,7 +179,7 @@ class BGPSpeaker(object):
                      enable_vpnv4=DEFAULT_CAP_MBGP_VPNV4,
                      enable_vpnv6=DEFAULT_CAP_MBGP_VPNV6,
                      next_hop=None, password=None, multi_exit_disc=None,
-                     site_of_origins=None):
+                     site_of_origins=None, is_route_server_client=False):
         """ This method registers a new neighbor. The BGP speaker tries to
         establish a bgp session with the peer (accepts a connection
         from the peer and also tries to connect to it).
@@ -210,12 +212,16 @@ class BGPSpeaker(object):
 
         ``site_of_origins`` specifies site_of_origin values.
         This parameter must be a list of string.
+
+        ``is_route_server_client`` specifies whether this neighbor is a
+        router server's client or not.
         """
         bgp_neighbor = {}
         bgp_neighbor[neighbors.IP_ADDRESS] = address
         bgp_neighbor[neighbors.REMOTE_AS] = remote_as
         bgp_neighbor[PEER_NEXT_HOP] = next_hop
         bgp_neighbor[PASSWORD] = password
+        bgp_neighbor[IS_ROUTE_SERVER_CLIENT] = is_route_server_client
         # v6 advertizement is available with only v6 peering
         if netaddr.valid_ipv4(address):
             bgp_neighbor[CAP_MBGP_IPV4] = enable_ipv4
@@ -284,8 +290,7 @@ class BGPSpeaker(object):
                  neighbors.CHANGES: attribute_param}
         call(func_name, **param)
 
-    def prefix_add(self, prefix, next_hop=None, route_dist=None,
-                   route_family=None):
+    def prefix_add(self, prefix, next_hop=None, route_dist=None):
         """ This method adds a new prefix to be advertized.
 
         ``prefix`` must be the string representation of an IP network
@@ -308,9 +313,19 @@ class BGPSpeaker(object):
         if route_dist:
             func_name = 'prefix.add_local'
             networks[ROUTE_DISTINGUISHER] = route_dist
+
+            rf, p = self._check_rf_and_normalize(prefix)
+            networks[ROUTE_FAMILY] = rf
+            networks[PREFIX] = p
+
+            if rf == vrfs.VRF_RF_IPV6 and netaddr.valid_ipv4(next_hop):
+                # convert the next_hop to IPv4-Mapped IPv6 Address
+                networks[NEXT_HOP] = \
+                    str(netaddr.IPAddress(next_hop).ipv6())
+
         call(func_name, **networks)
 
-    def prefix_del(self, prefix, route_dist=None, route_family=None):
+    def prefix_del(self, prefix, route_dist=None):
         """ This method deletes a advertized prefix.
 
         ``prefix`` must be the string representation of an IP network
@@ -327,10 +342,15 @@ class BGPSpeaker(object):
         if route_dist:
             func_name = 'prefix.delete_local'
             networks[ROUTE_DISTINGUISHER] = route_dist
+
+            rf, p = self._check_rf_and_normalize(prefix)
+            networks[ROUTE_FAMILY] = rf
+            networks[PREFIX] = p
+
         call(func_name, **networks)
 
     def vrf_add(self, route_dist, import_rts, export_rts, site_of_origins=None,
-                multi_exit_disc=None):
+                route_family=RF_VPN_V4, multi_exit_disc=None):
         """ This method adds a new vrf used for VPN.
 
         ``route_dist`` specifies a route distinguisher value.
@@ -342,13 +362,19 @@ class BGPSpeaker(object):
         ``site_of_origins`` specifies site_of_origin values.
         This parameter must be a list of string.
 
+        ``route_family`` specifies route family of the VRF.
+        This parameter must be RF_VPN_V4 or RF_VPN_V6.
         """
 
+        assert route_family in (RF_VPN_V4, RF_VPN_V6),\
+            'route_family must be RF_VPN_V4 or RF_VPN_V6'
+
         vrf = {}
         vrf[vrfs.ROUTE_DISTINGUISHER] = route_dist
         vrf[vrfs.IMPORT_RTS] = import_rts
         vrf[vrfs.EXPORT_RTS] = export_rts
         vrf[vrfs.SITE_OF_ORIGINS] = site_of_origins
+        vrf[vrfs.VRF_RF] = route_family
         call('vrf.create', **vrf)
 
     def vrf_del(self, route_dist):
@@ -362,12 +388,13 @@ class BGPSpeaker(object):
         vrf[vrfs.ROUTE_DISTINGUISHER] = route_dist
         call('vrf.delete', **vrf)
 
-    def vrfs_get(self):
+    def vrfs_get(self, format='json'):
         show = {}
         show['params'] = ['vrf', 'routes', 'all']
+        show['format'] = format
         return call('operator.show', **show)
 
-    def rib_get(self, family='ipv4'):
+    def rib_get(self, family='ipv4', format='json'):
         """ This method returns the BGP routing information in a json
         format. This will be improved soon.
 
@@ -376,6 +403,7 @@ class BGPSpeaker(object):
         """
         show = {}
         show['params'] = ['rib', family]
+        show['format'] = format
         return call('operator.show', **show)
 
     def _set_filter(self, filter_type, address, filters):
@@ -468,16 +496,48 @@ class BGPSpeaker(object):
         in_filter = call(func_name, **param)
         return in_filter
 
-    def bmp_start(self, host, port):
+    def bmp_server_add(self, address, port):
+        """This method registers a new BMP (BGP monitoring Protocol)
+        server. The BGP speaker starts to send BMP messages to the
+        server. Currently, only one BMP server can be registered.
+
+        ``address`` specifies the IP address of a BMP server.
+
+        ``port`` specifies the listen port number of a BMP server.
+        """
+
         func_name = 'bmp.start'
         param = {}
-        param['host'] = host
+        param['host'] = address
         param['port'] = port
         call(func_name, **param)
 
-    def bmp_stop(self, host, port):
+    def bmp_server_del(self, address, port):
+        """ This method unregister the registered BMP server.
+
+        ``address`` specifies the IP address of a BMP server.
+
+        ``port`` specifies the listen port number of a BMP server.
+        """
+
         func_name = 'bmp.stop'
         param = {}
-        param['host'] = host
+        param['host'] = address
         param['port'] = port
         call(func_name, **param)
+
+    @staticmethod
+    def _check_rf_and_normalize(prefix):
+        """ check prefix's route_family and if the address is
+        IPv6 address, return IPv6 route_family and normalized IPv6 address.
+        If the address is IPv4 address, return IPv4 route_family
+        and the prefix itself.
+
+        """
+        ip, masklen = prefix.split('/')
+        if netaddr.valid_ipv6(ip):
+            # normalize IPv6 address
+            ipv6_prefix = str(netaddr.IPAddress(ip)) + '/' + masklen
+            return vrfs.VRF_RF_IPV6, ipv6_prefix
+        else:
+            return vrfs.VRF_RF_IPV4, prefix
